/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/lino_czt.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 *********************************************************
 *                                                       *
 *           CHIRP-Z TRANSFORM ALGORITHM                 *
 *                                                       *
 *     THIS ROUTINE IMPLEMENTS THE CHIRP-Z TRANSFORM     *
 *     ALGORITHM.  USE OF THIS ALGORITHM PERMITS         *
 *     COMPUTATION OF THE DFT OF A GIVEN                 *
 *     INPUT SEQUENCE ALONG A MORE GENERAL CONTOUR IN    *
 *     THE Z-PLANE.                                      *
 *                                                       *
 *     REFERENCE:   RABINER, SCHAFER, AND RADER,         *
 *                IEEE TRANSACTIONS ON                   *
 *                AUDIO AND ELECTROACOUSTICS, VOL.       *
 *                AU-17, NO.2                            *
 *                                                       *
 *     VERSION 0.0 : DAVID A. ROBERTS, 7/87              *
 *                                                       *
 *********************************************************

 INPUT -     X: ARRAY CONTAINING (COMPLEX) SAMPLES
 DOLD: REAL SAMPLE-SPACING
 DNEW: DESIRED FOURIER SPACING
 2*NOLD+1: NUMBER OF INPUT SAMPLES
 2*NNEW+1: NUMBER OF FOURIER SAMPLES

 OUTPUT - ODATA: ARRAY CONTAINING FOURIER SAMPLES

 NOTE: ALL COMPLEX QUANTITIES ARE STORED IN (real,imag) FORM.
 */

//      parameter (pi = 3.141592654)
#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "snfft.h"
#include "consts.h"

#include "lino.h"

void lino_class::czt(REAL* x, REAL* odata, REAL dold, REAL dnew, INTEGER nold, INTEGER nnew)
{
	// bug 195 - correct array sizes - swr - 1/17/06
	REAL rl1, im1;
	REAL pdd;

	REAL phase;

	INTEGER n3[3];
	INTEGER L;
	INTEGER offset;

	INTEGER i;
	INTEGER ii;
	INTEGER k;

	offset = 2 * nold;
	pdd = Consts.pi * dold * dnew;
	L = 4;

	while (!(L >= (3 * nold + 1)))
	{
		L = L * 2;
	}

	n3[0] = L;
	n3[1] = 1;
	n3[2] = 1;

	// FORM Y(N) SEQUENCE - L VALUES

	i = 0;
	ii = offset;

	// bug 195 - correct array sizes - swr - 1/17/06
	REAL rl[nold + 2];
	REAL im[nold + 2];
	REAL y[2 * L];

	for (k = 0; k <= nold; k++)
	{
		phase = -pdd * k * k;
		rl[k] = (REAL) cos(phase);
		im[k] = (REAL) sin(phase);
		y[i] = rl[k] * x[ii] - im[k] * x[ii + 1];
		y[i + 1] = im[k] * x[ii] + rl[k] * x[ii + 1];
		i += 2;
		ii += 2;
	}

	for (k = 1; k <= 2 * (L - 2 * nold - 1); k++)
	{
		y[i] = 0.0;
		i++;
	}

	ii = 0;
	for (k = -nold; k <= -1; k++)
	{
		rl1 = rl[-k];
		im1 = im[-k];
		y[i] = rl1 * x[ii] - im1 * x[ii + 1];
		y[i + 1] = im1 * x[ii] + rl1 * x[ii + 1];
		i += 2;
		ii += 2;
	}

	// COMPUTE L-POINT FFT OF Y(N)

	snfft(y, n3, 1);

	// FORM V(N) SEQUENCE - L VALUES

	REAL v[2 * L];   // bug 195 - correct array sizes - swr - 1/17/06

	i = 0;
	for (k = 0; k <= L / 2; k++)
	{
		phase = pdd * k * k;
		v[i] = (REAL) cos(phase);
		v[i + 1] = (REAL) sin(phase);
		i += 2;
	}

	ii = i - 4;
	for (k = -(L / 2) + 1; k <= -1; k++)
	{
		v[i] = v[ii];
		v[i + 1] = v[ii + 1];
		ii -= 2;
		i += 2;
	}

	// COMPUTE L-POINT FFT OF V(N)

	snfft(v, n3, 1);

	// MULTIPLY THE ARRAYS Y AND V, POINT-BY-POINT; CALL THE RESULT G(N)

	REAL g[2 * L];   // bug 195 - correct array sizes - swr - 1/17/06
	i = 0;
	for (k = 0; k < L; k++)
	{
		g[i] = v[i] * y[i] - v[i + 1] * y[i + 1];
		g[i + 1] = v[i + 1] * y[i] + v[i] * y[i + 1];
		i += 2;
	}

	// COMPUTE L-POINT INVERSE FFT OF G(N)

	snfft(g, n3, -1);

	// MULTIPLY G(N) BY (dold)*exp(-i*pi*dold*dnew*k*k)
	// MAKE SURE THAT ODATA IS IN PROPER FORMAT (DC-VALUE IN MIDDLE)

	i = 2 * nnew;
	ii = 0;
	for (k = 0; k <= nnew; k++)
	{
		rl1 = dold * rl[k];
		im1 = dold * im[k];
		odata[i] = g[ii] * rl1 - g[ii + 1] * im1;
		odata[i + 1] = g[ii + 1] * rl1 + g[ii] * im1;
		i += 2;
		ii += 2;
	}

	i = 0;
	ii = 2 * L - 2 * nnew;
	for (k = -nnew; k <= -1; k++)
	{
		rl1 = dold * rl[-k];
		im1 = dold * im[-k];
		odata[i] = g[ii] * rl1 - g[ii + 1] * im1;
		odata[i + 1] = g[ii + 1] * rl1 + g[ii] * im1;
		i += 2;
		ii += 2;
	}

	return;
}
