/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/ray.cpp $
 $LastChangedRevision: 91 $
 $Date: 2014-07-02 17:34:27 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 RAY RETURNS A LIST OF POINTS THAT LIE IN THE N RAY-TH RAY TAKEN
 AT ANGLE THETA AND WITH RAY WIDTHS DX AND DY MEASURED IN
 THE X AND Y DIRECTIONS.
 NUM IS THE NUMBER OF POINTS IN THE RAY.
 LIST IS THE SET OF POINTS RETURNED.  FOR A POINT WITH COORDINATES
 (I,J), LIST = (J-1)*WIDTH + I

 Algorithm rewritten and simplified as part of bug 82 - swr 2/20/05
 */

#include <cstdio>
#include <cmath>
#include <stdlib.h>

#include "blkdta.h"
#include "geom.h"
#include "uiod.h"

#include "posit.h"

#include "ray.h"

void ray(INTEGER nproj, INTEGER nray, INTEGER* list, REAL* weight,
		INTEGER* numb, REAL* snorm)
{
	int i, j;

	//INTEGER lo,hi;    // bug82 - swr 2/20/05

	static INTEGER lastp = -1; // made static to match the intent of the Fortran code. Lajos, Feb 10, 2005

	REAL ax;
	REAL ay;
	REAL costh;
	REAL sinth;
	static REAL dee;
	static REAL pmid; // made static to match the intent of the Fortran code. Lajos, Feb 10, 2005
	static REAL sin_1;    // bug82 start - swr 2/20/05
	static REAL cos_1;
	static INTEGER inci; // made static to match the intent of the Fortran code. Lajos, Feb 10, 2005
	static INTEGER incj; // made static to match the intent of the Fortran code. Lajos, Feb 10, 2005
	//INTEGER iscrpt;    // bug82 end - swr 2/20/05
	REAL pix2;
	static INTEGER midpix;    // bug82 - swr 2/20/05
	static INTEGER nrlo, nrhi;

	// bug 168 - swr - 9/24/05
	if (GeoPar.div)
	{
		fprintf(output,
				"\n         ***** ray is not usable with divergent geometry\n");
		exit(1);
	}

	*numb = 0;
	posit(nproj, nray, &ax, &ay, &costh, &sinth);
	//nr = (REAL) (nray - GeoPar.midray);    // bug82 - swr 2/20/05

	if (nproj != lastp)
	{

		// INITIALIZATION FOR A NEW PROJECTION ANGLE

		lastp = nproj;
		if (GeoPar.uni)
			dee = GeoPar.pinc;
		if (GeoPar.vri)
			dee = GeoPar.pinc * ((REAL) MAX0(fabs(costh), fabs(sinth)));
		pmid = (GeoPar.nelem + 1) / ((REAL) 2.0);

		if (fabs(sinth) < fabs(costh))
		{
			sin_1 = sinth;
			cos_1 = costh;
			inci = 1;
			incj = GeoPar.nelem;
		}
		// FOR ANGLES THAT ARE CLOSER TO THE Y-AXIS THAN TO THE X-AXIS,
		// THE FOLLOWING EFFECTIVELY PERFORMS A ROTATION ABOUT THE LINE X=-Y.
		else
		{
			sin_1 = costh;
			cos_1 = sinth;
			inci = GeoPar.nelem;
			incj = 1;
		}
		midpix = GeoPar.midpix - 1;    // bug82 - swr 2/20/05
		// find limits of the rays that can intersect the reconstruction region
		REAL halfw = midpix * GeoPar.pixsiz;
		INTEGER nr = (INTEGER) ceil(halfw * (fabs(costh) + fabs(sinth)) / dee);
		nrlo = GeoPar.midray - nr;
		nrhi = GeoPar.midray + nr;
	}

	if (nray >= nrlo && nray <= nrhi)
	{

		// DETERMINE THE POINTS THAT LIE IN THE RAY
		// The equation of a line making an angle atan2(sin_1, cos_1)
		// with the x axis and a perpendicular distance r from the origin
		// is given by y*cos_1 - x * sin_1 = r.  swr - 2/20/05

		// find the distance the two edges of the strip are from the origin
		REAL r1 = ((REAL) (nray - GeoPar.midray) - 0.5) * dee;
		REAL r2 = ((REAL) (nray - GeoPar.midray) + 0.5) * dee;

		for (i = -midpix; i <= midpix; ++i)
		{
			// for each column of pixels, find the two edges of the strip
			REAL x = i * GeoPar.pixsiz;
			REAL y1 = (r1 + x * sin_1) / cos_1;
			REAL y2 = (r2 + x * sin_1) / cos_1;
			// swap if necessary
			if (y1 > y2)
			{
				REAL t = y1;
				y1 = y2;
				y2 = t;
			}
			// convert to pixel coordinates
			INTEGER jlo = MAX0((INTEGER)ceil(-y2 / GeoPar.pixsiz), -midpix);
			INTEGER jhi = MIN0((INTEGER)ceil(-y1 / GeoPar.pixsiz), midpix + 1);
			// add to the list
			for (j = jlo; j < jhi; ++j)
			{
				list[*numb] = (j + midpix) * incj + (i + midpix) * inci;
				if (list[*numb] < 0 || list[*numb] >= GeoPar.area)
					printf("\nray: error pixel out of range = %d", list[*numb]);
				++*numb;
			}
		}
		// end of new algorithm - swr 2/20/05
	}

	pix2 = GeoPar.pixsiz * GeoPar.pixsiz;

	for (i = 0; i < *numb; i++)
	{
		weight[i] = pix2;
	}

	*snorm = pix2 * pix2 * (*numb);
	if (trace > 6)
	{
		fprintf(output,
				"\n          ray     np = %5i  nr = %5i  numb = %5i  snorm = %10.4f",
				nproj, nray, *numb, *snorm);
	}

	if ((*numb > 0) && (trace > 9))
	{
		for (i = 0; i < *numb; i++)
		{
			if (i % 10 == 0)
				fprintf(output, "\n                  ");
			fprintf(output, "%6i", list[i]);
		}
	}

	return;
}
