/** @file chooseRanges.cpp
    @package snark14Display
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

/** chooseRanges.cpp
    @author deniz 

*/
#include "chooseRanges.hpp"
#include "displaylines.hpp"
#include "SnarkDisplay.hpp"

#include "plot_t.hpp"
#include "line_real_set_t.hpp"
#include "line_real_t.hpp"

#define EPSILON 0.00000001

/** 
 *  Constructs a chooseRangesDialog which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'fl', initial topleft position to (x,y)
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
chooseRangesDialog::chooseRangesDialog( QWidget* parent, const char* name, bool modal, WFlags fl , int x, int y,
			    unsigned ee_index, bool forPBP, unsigned PBPYVar, globalYVar_t GlobalYVar, 				    int minX, int maxX, double minY, double maxY)  : QDialog( parent, name, modal, fl )
{
  this->ee_index = ee_index;
  this->forPBP   = forPBP;
  this->PBPYVar  = PBPYVar;
  this->GlobalYVar = GlobalYVar;

  QPoint topleft;
  if(forPBP) {
    YVarName = tr("e(%1)").arg(PBPYVar);
  } else {
    switch(GlobalYVar) {
    case AVERAGE:   YVarName = "Average";  break;
    case DISTANCE:  YVarName = "Distance"; break;
    case RELERR:    YVarName = "Relative Error";   break;
    case VARIANCE:  YVarName = "Variance"; break;
    case STDDEV:    YVarName = "Standard Deviation";   break;
    case RESIDUAL:  YVarName = "Residual"; break;
    case KULLBACK:  YVarName = "Kullback-Leibler Distance"; break;
    default: std::cerr << "Invalid GlobalYVar in chooseRangesDialog ctor!" << std::endl;
    }
  }

  if ( !name )
    setName( "chooseRangesDialogWindow" );

  fullname = tr("%1 :: %2").arg(name).arg(YVarName); // weird QT syntax, don't sweat it

  setCaption( fullname );
  topleft.setX(x);
  topleft.setY(y);
  this->move(topleft);
  
  QBoxLayout *topLayout = new QVBoxLayout( this );
  
  minx = new QLineEdit( this, "minx" );
  maxx = new QLineEdit( this, "maxx" );
  miny = new QLineEdit( this, "miny" );
  maxy = new QLineEdit( this, "maxy" );
  
  QBoxLayout* x_extrema_layout = new QHBoxLayout( topLayout );
  minxlabel = new QLabel( this, "minxlabel" );
  minxlabel->setText( tr( "Min Iter:" ) );
  maxxlabel = new QLabel( this, "maxxlabel" );
  maxxlabel->setText( tr( "Max Iter:" ) );
  x_extrema_layout->addWidget(minxlabel);
  x_extrema_layout->addWidget(minx);
  x_extrema_layout->addWidget(maxxlabel);
  x_extrema_layout->addWidget(maxx);
  
  QBoxLayout* y_extrema_layout = new QHBoxLayout( topLayout );
  minylabel = new QLabel( this, "minxlabel" );
  minylabel->setText( tr( "Min Y:" ) );
  maxylabel = new QLabel( this, "maxxlabel" );
  maxylabel->setText( tr( "Max Y:" ) );
  y_extrema_layout->addWidget(minylabel);
  y_extrema_layout->addWidget(miny);
  y_extrema_layout->addWidget(maxylabel);
  y_extrema_layout->addWidget(maxy);    

  minx->setText( tr("%1") . arg( minX ) );
  maxx->setText( tr("%1") . arg( maxX ) );
  miny->setText( tr("%1") . arg( minY ) );
  maxy->setText( tr("%1") . arg( maxY ) );

  QString plottext(tr("plot %1").arg(YVarName));
  plotbutton = new QPushButton( this, "plotbutton" );
  plotbutton->setText( plottext );
  plotbutton->setEnabled(true);
  QObject::connect(plotbutton,SIGNAL(clicked()),this,SLOT(proceed()));
  topLayout->addWidget( plotbutton );
  
  closebutton = new QPushButton( this, "closebutton" );
  closebutton->setText( tr( "Close Window" ) );
  QObject::connect(closebutton,SIGNAL(clicked()),this,SLOT(closeWindow()));
  
  topLayout->addWidget( closebutton );
  topLayout->activate();
}

/**  
 *  Destroys the object and frees any allocated resources
 */
chooseRangesDialog::~chooseRangesDialog()
{
  // no need to delete child widgets, Qt does it all for us
}

void chooseRangesDialog::closeWindow()
{
  //  accept();
  reject();
}

void chooseRangesDialog::closeEvent(QCloseEvent * e)
{
  e->accept();
}

/** Called when plotbutton "Plot" is clicked:
    based on the sameExtremaForAllCheckBox state,
    either spawns plots or new dialog boxes.
    Does some checks before sending params to plots (no garbage out).
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
void chooseRangesDialog::proceed()
{
  if(std::verbose>=2) {
    std::cout << "in chooseRangesDialog::proceed()" << std::endl;
  }
  bool complain = false;
  // attempt to call plot windows after error checking
  bool ok = true;
  int minXVal = minx->text().toInt(&ok);
  if((!ok) || (minXVal < 0) ) complain=true; // allow minXVal to be 0
  int maxXVal = maxx->text().toInt(&ok);
  if(!ok) complain=true;
  if(maxXVal <= 0) complain=true;
  if(minXVal >= maxXVal) complain=true;
  double minYVal = miny->text().toDouble(&ok);
  if(!ok) complain=true;
  double maxYVal = maxy->text().toDouble(&ok);
  if(!ok) complain=true;
  // allow negative values for minYVal, even though there are no negative ones in data
  if(maxYVal <= 0) complain=true;
  if(minYVal >= maxYVal) complain=true;
  if(complain) {
    std::cerr << "Invalid combination of X and Y parameters." << std::endl;
  } else { // passed error checks
    if(std::verbose>=2) {
      std::cout << "Plotting with iter: [" << minXVal 
		<< "," << maxXVal <<	  "]; y: [" << minYVal << "," 
		<< maxYVal << "]" << std::endl;
    }
    ///   spawnPlots(true, minXVal, maxXVal, minYVal, maxYVal);
    //    std::cout << "PLOTTING!" << std::endl;
    line_real_set_t LRS(minXVal, maxXVal, minYVal, maxYVal);
    ///    int PMWidth  = 550; // pixmap width, etc
    ///    sd_line_t::WIDTH  = 550;
    int PMWidth = sd_line_t::WIDTH;
    ///    int PMHeight = 450;
    ///    sd_line_t::HEIGHT = 450;
    int PMHeight = sd_line_t::HEIGHT;
    QPixmap* myPixmapP = new QPixmap(PMWidth,PMHeight);
    myPixmapP->fill( Qt::white );
    QPainter* myPainterP = new QPainter;
    myPainterP->begin(myPixmapP);
    // do the thing with the plot
    if(forPBP) {
      myEES.addPBPVarLines(LRS, PBPYVar, ee_index);
    } else {
      myEES.addGlobVarLines(LRS, GlobalYVar, ee_index);
    }
    plot_t* myPlotterP = new plot_t(LRS);
    ///    plot_t myPlotter(LRS);       // now combine LRS with myPainterP
    myPlotterP->setName(fullname);
    QString xAxisLabel = "Iteration"; // as in all evals
    myPlotterP->setXAxisName(xAxisLabel);
    myPlotterP->setYAxisName(YVarName); // YVarName is an instance variable
    myPlotterP->plot(*myPainterP);
    displaylineswindow *dwp = new displaylineswindow( this , fullname, myPixmapP, false, 0, 
						      -20+this->pos().x(), 20+this->pos().y() );
    dwp->show(); // and display!
  }
} // --chooseRangesDialog::proceed()
